;==========================================================
;
; Speaker Sound Playing Routines for Turbo Assember
; Copyright (c) 1991 Lane Roath & Ideas From the Deep
;
;==========================================================

		IDEAL
		MODEL	small,C

		DATASEG
;----------------------------------------------------------
;
;offsets into .SSE file pointed to by SoundData & PARA aligned
;sound records each take up 16 bytes, w/first record the file header

		PUBLIC	SoundData,soundmode,dontplay

s_start		=	0
s_priority	=	2
s_samples	=	3
s_name		=	4

SPKactive	dw	0	;set non zero when started

SoundData	dd	?	;two word pointer to SPKR file, PARA aligned
soundmode	dw	1	;0=nosound, 1=SPKR, 2= adlib...
OldInt8		dd	?	;StartupSPK saves here, Shutdown restores
ExtraInts	db	?	;number of PlaySPKR's to a regular int 8
Intcount	db	?	;counter for extraints, call OldInt8 at 0
sndspeed	dw	?	;timer count speed

SndPtr		dw	?	;Pointer to frequency of current sound
SndPriority	db	?	;current sound's priority

dontplay	dw	0	;set to 1 to avoid all interrupt and timer stuff

;----------------------------------------------------------

		CODESEG

;=======================================================================
;
; StartupSound
;
; Sets up the new INT 8 ISR and various internal pointers.
; Assumes that the caller has pointed SOUNDDATA to sound buffer
;

PROC StartupSound
	PUBLIC	StartupSound

	cmp	[dontplay],0
	je	@@dowork
	ret
@@dowork:
	cmp	[SPKactive],0	;see if library is active
	jne	@@started

@@start:
	call	NEAR PTR StopSound	;make sure nothing is playing

	mov	ax,3508h	;call bios to get int 8
	int	21h
	mov	[WORD oldint8],bx
	mov	ax,es
	mov	[WORD oldint8+2],ax

	mov	ax,1
	mov	[extraints],al		;the timer is still going at the
	mov	[intcount],al		;normal rate now

	push	ds

	push	cs
	pop	ds
	lea	dx,[UpdateSPKR]
	mov	ax,2508h	;call bios to set int 8
	int	21h

	pop	ds

	inc	[SPKactive]	;sound routines are now active

@@started:
	mov	ax,1
	mov	[soundmode],ax ;set soundmode to SPKR
	ret

ENDP

;=======================================================================
;
; ShutdownSound
;

PROC	ShutdownSound
	PUBLIC ShutdownSound

	cmp	[dontplay],0	;play sound?
	je	@@dowork
	ret
@@dowork:
	mov	al,36h		;tell the timer chip we are going to
	out	43h,al		;change the speed of timer 0
	mov	al,0		;system expects 0000 for rate
	out	40h,al		;low
	out	40h,al		;high

	cmp	[SPKactive],0
	je	@@done		;sound library wasn't started...

	push	ds

	mov	dx,[WORD Oldint8]
	mov	ax,[WORD Oldint8+2]
	mov	ds,ax
	mov	ax,2508h	;call bios to set int 8
	int	21h

	pop	ds

	mov	[SPKactive],0	;sound routines are now inactive

	in	al,61h		;get peripheral (speaker) port value
	and	al,11111101b	;turn speaker off
	out	61h,al

@@done:
	ret

ENDP

;=========================================================================
;
; PlaySound (soundnum)
;
; If the sound's priority is >= the current priority, SoundPtr, SndPriority,
; and the timer speed are changed
;

PROC	PlaySound playnum:WORD
	PUBLIC PlaySound

	cmp	[dontplay],0
	je	@@dowork
	ret
@@dowork:
	mov	ax,[playnum]	;index into the sound headers
	push	si

	mov	si,ax
	shl	si,1
	shl	si,1
	shl	si,1
	shl	si,1

	mov	ax,[WORD SoundData+2]
	mov	es,ax		;point es: to the spkr file

	mov	al,[es:si+s_Priority]	;priority table (one byte each)
	cmp	al,[SndPriority]
	jb	@@playdone	;current sound has higher priority
	mov	[SndPriority],al
	mov	ax,[es:si+s_Start]	;offset in .SPK file
	mov	[SndPtr],ax	;store it in the sound playing table

	mov	bl,[es:si+s_samples]	;samples / regular timer tick (1-255)
	mov	[extraints],bl	;an OldInt8 will be called after this
	mov	[intcount],bl	;many UpdateSPKR times have been called

	cmp	bl,1		;sample rate of 0 or 1 = 0000 for timer
	ja	@@oktodiv
	xor	bx,bx
	jmp	@@settimer

@@oktodiv:
	xor	bh,bh
	xor	ax,ax
	mov	dx,1
	div	bx		;$10000 / samples = timer rate

	mov	bx,ax
	mov	[sndspeed],bx	;save off the timer rate

@@settimer:
	mov	al,36h		;tell the timer chip we are going to
	out	43h,al		;change the speed of timer 0

	mov	al,bl		;low byte of sample rate
	out	40h,al
	mov	al,bh		;high byte of sample rate
	out	40h,al

@@playdone:
	pop	si
	ret

ENDP


;======================================================================
;
; StopSound
;

PROC	StopSound
	PUBLIC	StopSound

	cmp	[dontplay],0
	je	@@dowork
	ret
@@dowork:
	xor	ax,ax		;set to 0
	mov	[SndPtr],ax
	mov	[SndPriority],al
	mov	[sndspeed],ax

	in	al,61h		;get peripheral (speaker) port value
	and	al,11111101b	;turn speaker off
	out	61h,al

	mov	al,36h		;tell the timer chip we are going to
	out	43h,al		;change the speed of timer 0

	mov	al,0		;back to normal clock speed
	out	40h,al
	out	40h,al

	inc	al
	mov	[extraints],al  ;one bios int / int8
	mov	[intcount],al

	ret
ENDP

;======================================================================
;
; WaitendSound
; Just waits around until the current sound stops playing
;

PROC	WaitEndSound
	PUBLIC WaitEndSound

	cmp	[dontplay],0
	je	@@dowork
	ret
@@dowork:
	pushf
	call FAR PTR UpdateSPKR	;in case a sound was just started and hasn't
							;been hit by an INT yet
@@wait:
	mov	ax,[sndptr]
	or	AX,AX		;when the ptr is 0, nothing is on
	jne	@@wait

	ret

ENDP

;=========================================================================
;
; UpdateSPKR
; only called by interrupt $8!
;

PROC	UpdateSPKR FAR

	push	ax
	push	bx
	push	cx
	push	si
	push	ds
	push	es

	mov	ax,_DATA
	mov	ds,ax		;ds to this data segment
	mov	ax,[WORD SoundData+2]
	mov	es,ax		;es to sound file

	mov	al,20h
	out	20h,al		;we got the interrupt here

	dec	[intcount]	;see if it is time for a BIOS int 8...
	jne	@@dosounds

	mov	al,[extraints]
	mov	[intcount],al	;reset interrupt counter

	pushf			;so the IRET from bios returns right
	call	[OldInt8]	;call the old BIOS timer routine

@@dosounds:
;
; play the speaker
;
	mov	si,[SndPtr]
	cmp	si,0
	je	@@nosound	;nothing playing

	mov	bx,[es:si]
	inc	[SndPtr]
	inc	[SndPtr]

	cmp	bx,0
	je	@@nosound	;a zero frequency is no sound, but don't stop

	cmp	bx,-1		;a -1 frequency is end of sound
	jne	@@playfreq

	call	StopSound
	jmp	SHORT @@doneplay

@@nosound:
	in	al,61h		;get peripheral (speaker) port value
	and	al,11111100b	;turn speaker off
	out	61h,al
	jmp	@@doneplay

@@playfreq:
	cmp	[soundmode],0	;if sounds off, don't play anything
	je	@@nosound

	mov	al,10110110b	;write to channel 2 (speaker) timer
	out	43h,al
	mov	al,bl
	out	42h,al		;low byte
	mov	al,bh
	out	42h,al		;high byte

	in	al,61h		;get peripheral (speaker) port value
	or	al,00000011b	;turn speaker on to timer
	out	61h,al

@@doneplay:
	pop	es
	pop	ds
	pop	si
	pop	cx
	pop	bx
	pop	ax

	iret


ENDP

END